###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from obt import dep, command, pathtools, path
###############################################################################
class nlohmannjson(dep.StdProvider):
  VERSION ="v3.11.3"
  NAME = "nlohmannjson"
  def __init__(self):
    super().__init__(nlohmannjson.NAME)
    self.declareDep("cmake")    
    self._builder = dep.CMakeBuilder(nlohmannjson.NAME)
    self._builder.setCmVars({
        "CMAKE_BUILD_TYPE": "RELEASE",
        "BUILD_EXAMPLES": "ON"
    })
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=nlohmannjson.NAME,
                             repospec="nlohmann/json",
                             revision=nlohmannjson.VERSION,
                             recursive=True)
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.include()/"nlohmann"/"json.hpp").exists()




###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import platform, os, pathlib,sys
import multiprocessing

SYSTEM = platform.system()
IsOsx = (SYSTEM=="Darwin")
IsDarwin = (SYSTEM=="Darwin")
IsIrix = (SYSTEM=="IRIX64")
IsLinux = (SYSTEM=="Linux")
IsIx = IsLinux or IsOsx or IsIrix
IsX86_64 = platform.machine()=="x86_64"
IsAARCH64 = (platform.machine()=="aarch64") or (platform.machine()=="arm64")
IsX86_32 = platform.machine()=="i686"
IsAppleSilicon = IsAARCH64 and IsDarwin
file_path = os.path.realpath(__file__)
this_dir = pathlib.Path(os.path.dirname(file_path))

###############################################################################

def description():
  import obt.module
  import obt.path
  hostinfo_dir = obt.path.obt_modules_base()/"host"
  the_module = None
  if IsOsx:
    machine = platform.machine()
    if machine == "arm64":
       machine = "aarch64"
    identifier = machine+"-"+"macos"
    hi_name = hostinfo_dir/("%s.py"%identifier)
    the_module = obt.module.instance(identifier,hi_name)
  elif IsLinux:
    identifier = platform.machine()+"-"+"linux"
    hi_name = hostinfo_dir/("%s.py"%identifier)
    the_module = obt.module.instance(identifier,hi_name)
  if the_module != None:
    return the_module.hostinfo()

###############################################################################

class enuminterface:
  def __init__(self):
    self.subdir = "host"
  def tryAsModule(self,item,pth):
    identifier = item.replace(".py","")
    m = obt.module.instance(identifier,pth)
    if hasattr(m,"hostinfo"):
      return m
    else:
      return None

###############################################################################

def enumerate():
  iface = enuminterface()
  return obt.module.enumerate_simple(iface)

###############################################################################

def _TryGentoo():
	portage_exists = os.path.exists("/etc/portage")
	return portage_exists

###############################################################################

def _TryDebian():
	apt_exists = os.path.exists("/etc/apt")
	return apt_exists

###############################################################################

IsGentoo = _TryGentoo()
IsDebian = _TryDebian()

###############################################################################

if "OBT_NUM_CORES" in os.environ:
  NumCores = int(os.environ["OBT_NUM_CORES"])
else:
  NumCores = multiprocessing.cpu_count()

if IsLinux:
    PlatformId = "ix"
elif IsOsx:
    PlatformId = "osx"
else:
    PlatformId = "unknown"

###############################################################################



#!/usr/bin/env python3

import os, sys, subprocess, argparse

from obt import command, path, pathtools, sdk, dep

parser = argparse.ArgumentParser(description='obt.build dep builder')
parser.add_argument('--gencert', action="store_true", help='generate ios certificate' )
parser.add_argument('--build', action="store_true", help='build...' )

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

args = parser.parse_args()

##############################################

prefix = path.subspace_root()/"xros"

command.run(["rm","-rf",prefix/"builds"],do_log=True)

pathtools.ensureDirectoryExists(prefix)
pathtools.ensureDirectoryExists(prefix/"builds")
pathtools.ensureDirectoryExists(prefix/"lib")
pathtools.ensureDirectoryExists(prefix/"bin")
pathtools.ensureDirectoryExists(prefix/"conan")
os.chdir(prefix)

##############################################

CERTNAME = "APHIDSYSC"

def generate_csr(output_path, certificate_name):
    """Generate a Certificate Signing Request (CSR) and add key to keychain."""
    try:
        # Use openssl to generate a CSR and key
        subprocess.run(["openssl", "req", "-new", "-newkey", "rsa:2048", "-nodes", #
                        "-keyout", f"{output_path}/{certificate_name}.key", # 
                        "-out", f"{output_path}/{certificate_name}.csr"], #
                        check=True)
        # Add the key to the keychain
        subprocess.run(["security", "import", f"{output_path}/{certificate_name}.key", "-k", "login.keychain", "-T", "/usr/bin/codesign"],
                        check=True)
        print("CSR generated and key added to keychain successfully.")
    except subprocess.CalledProcessError as e:
        print(f"Error generating CSR and adding key to keychain: {e}")
        raise

if args.gencert:
  print("Generating provisioning certificate...")
  generate_csr(str(prefix), CERTNAME )
  print("Certificate generated successfully.")
  print("Now, upload the certificate signing request to the Apple Developer portal to create a certificate.")
  sys.exit(0)


##############################################

TEMP_PATH = path.temp()
XROS_SDK = sdk.descriptor("aarch64","xros")
SDK_DIR = XROS_SDK._sdkdir
SDK_VER = XROS_SDK._sdkver
the_environ = {
  "OBT_SUBSPACE_BUILD_DIR": prefix/"builds",
  "OBT_SUBSPACE_LIB_DIR": prefix/"lib",
  "OBT_SUBSPACE_DIR": prefix,
  "OBT_SUBSPACE_BIN_DIR": prefix/"bin",
  "XROS_PREFIX": prefix,
  "XROS_SDK_DIR": SDK_DIR,
  "XROS_SDK_VER": SDK_VER,
  "XROS_CLANG_PATH": XROS_SDK._clang_path,
  "XROS_CLANGPP_PATH": XROS_SDK._clangpp_path,
  "OBT_SUBSPACE": "xros",
  "OBT_TARGET": "aarch64-xros",
  "CONAN_HOME": prefix/"conan",
  "CONAN_USER_HOME": prefix/"conan",
  "CONAN_CACHE_DIR": "~/.obt-global/conan2"
}     

##############################################
# generate cmake toolchain
##############################################

cmake_tc = """
set(XROS_SDK_PATH $ENV{XROS_SDK_DIR})
set(CMAKE_SYSTEM_NAME visionOS)

# Specify the architectures to build for (e.g., arm64 for actual devices)
set(CMAKE_OSX_ARCHITECTURES arm64)

# Specify the minimum xros deployment target
set(CMAKE_OSX_DEPLOYMENT_TARGET $ENV{XROS_SDK_VER})

# Specify the path to the compiler
set(CMAKE_C_COMPILER $ENV{XROS_CLANG_PATH})
set(CMAKE_CXX_COMPILER $ENV{XROS_CLANGPP_PATH})
set(CMAKE_INSTALL_PREFIX $ENV{OBT_SUBSPACE_BUILD_DIR})
"""

tc_output = prefix/"xros.toolchain.cmake"
with open(tc_output,"w") as f:
  f.write(cmake_tc)

##############################################
# generate conanfile
##############################################

# missing 
# glfw
# lexertl14
# parsertl14
# easyprof 
# rtmidi 
# cpppeglib 
# klein 
# portaudio 

# not working
#libigl/2.3.0
#openimageio/2.5.6
#rapidjson/cci.20230929 <incompatible with assimp/5.2.2>
#openblas/0.3.26 

lexertl = dep.instance("lexertl14")
print(lexertl)
#print(dir(lexertl))
print(lexertl._conanfile)

lexertl14_dir = prefix / "conan" / "lexertl14"
lexertl14_dir.mkdir(parents=True, exist_ok=True)  # Ensure the directory exists
lrtl_output = lexertl14_dir / "conanfile.py"
with open(lrtl_output, "w") as f:
    f.write(lexertl._conanfile)  # Assuming lexertl._conanfile is the recipe text
os.chdir(str(lexertl14_dir))
#conan export . lexertl14/tweakoz-obt@user/channel
command.run(["conan","export",".",
             "--user=user",
             "--channel=channel"], #
             environment=the_environ,do_log=True ) 

conanfile = """
[requires]

[generators]
CMakeDeps
CMakeToolchain
"""

"""
zlib/1.2.11
boost/1.81.0
assimp/5.2.2
ptex/2.4.2
openexr/3.2.1
rapidjson/cci.20220822
eigen/3.4.0
glm/cci.20230113
bullet3/3.25
zeromq/4.3.5
cppzmq/4.10.0
sigslot/1.2.2 
moltenvk/1.2.2
lexertl14/tweakoz-obt
"""

cf_output = prefix/"conanfile.txt"
with open(cf_output,"w") as f:
  f.write(conanfile)

##############################################
# generate conan profile for ios
##############################################

conan_host_profile = """
[settings]
os=visionOS
os.version=1.0
arch=armv8
compiler=apple-clang
compiler.version=15.0
compiler.libcxx=libc++
build_type=Release
os.sdk=xros
"""

cp_output = prefix/"xros.host.profile"
with open(cp_output,"w") as f:
  f.write(conan_host_profile)

##############################################

conan_build_profile = """
[settings]
os=Macos
arch=armv8  
compiler=apple-clang
compiler.version=15.0
compiler.libcxx=libc++
build_type=Release
"""

cp_output = prefix/"xros.build.profile"
with open(cp_output,"w") as f:
  f.write(conan_build_profile)

##############################################
  
cmd =  ["conan","install","."]
cmd += [f"--profile:host={prefix}/xros.host.profile"]
cmd += [f"--profile:build={prefix}/xros.build.profile"]
cmd += ["--build=missing"]

command.run(cmd,environment=the_environ,do_log=True)

##############################################

APPID = "com.example.minimalapp"
#BUNDLE_NAME = "MYAPP"

##############################################

cmake_lists_content = """
cmake_minimum_required(VERSION 3.15)
project(xros_minimal_app LANGUAGES CXX OBJCXX)
set(CMAKE_OSX_SYSROOT %s CACHE PATH "The xrOS SDK sysroot")
set(CMAKE_TOOLCHAIN_FILE %s)
#include(${CMAKE_BINARY_DIR}/conan_toolchain.cmake)
include(${CMAKE_CURRENT_SOURCE_DIR}/xros.toolchain.cmake)

find_package(ZLIB REQUIRED)

add_executable(${PROJECT_NAME} main.mm)

set_target_properties(${PROJECT_NAME} PROPERTIES
    XCODE_ATTRIBUTE_DEVELOPMENT_TEAM "$ENV{DEVELOPMENT_TEAM}"
    XCODE_ATTRIBUTE_CODE_SIGN_IDENTITY "iPhone Developer"
    XCODE_ATTRIBUTE_CODE_SIGN_STYLE "Automatic"
    XCODE_ATTRIBUTE_PRODUCT_BUNDLE_IDENTIFIER "%s"
    MACOSX_BUNDLE_GUI_IDENTIFIER "%s"
    MACOSX_BUNDLE_INFO_PLIST ${CMAKE_CURRENT_SOURCE_DIR}/Info.plist
)

find_library(UIKIT_FRAMEWORK UIKit)
target_link_libraries(${PROJECT_NAME} PRIVATE "${UIKIT_FRAMEWORK}")
#target_link_libraries(${PROJECT_NAME} PRIVATE ZLIB::ZLIB)
""" % (SDK_DIR,str(tc_output),APPID,APPID)

with open(prefix / "CMakeLists.txt", "w") as f:
    f.write(cmake_lists_content)

##############################################

info_plist_content = """
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
	<key>BuildMachineOSBuild</key>
	<string>23C71</string>
	<key>CFBundleDevelopmentRegion</key>
	<string>en</string>
	<key>CFBundleExecutable</key>
	<string>xros_minimal_app</string>
	<key>CFBundleIdentifier</key>
	<string>com.example.minimalapp</string>
	<key>CFBundleInfoDictionaryVersion</key>
	<string>6.0</string>
	<key>CFBundleName</key>
	<string>XROS_TEST</string>
	<key>CFBundlePackageType</key>
	<string>APPL</string>
	<key>CFBundleShortVersionString</key>
	<string>1.0</string>
	<key>CFBundleSupportedPlatforms</key>
	<array>
		<string>XRSimulator</string>
	</array>
	<key>CFBundleVersion</key>
	<string>1</string>
	<key>DTCompiler</key>
	<string>com.apple.compilers.llvm.clang.1_0</string>
	<key>DTPlatformBuild</key>
	<string>21N301</string>
	<key>DTPlatformName</key>
	<string>xrsimulator</string>
	<key>DTPlatformVersion</key>
	<string>1.0</string>
	<key>DTSDKBuild</key>
	<string>21N301</string>
	<key>DTSDKName</key>
	<string>xrsimulator1.0</string>
	<key>DTXcode</key>
	<string>1520</string>
	<key>DTXcodeBuild</key>
	<string>15C500b</string>
	<key>MinimumOSVersion</key>
	<string>1.0</string>
	<key>UIApplicationSceneManifest</key>
	<dict>
		<key>UIApplicationPreferredDefaultSceneSessionRole</key>
		<string>UIWindowSceneSessionRoleVolumetricApplication</string>
		<key>UIApplicationSupportsMultipleScenes</key>
		<true/>
		<key>UISceneConfigurations</key>
		<dict/>
	</dict>
	<key>UIDeviceFamily</key>
	<array>
		<integer>7</integer>
	</array>
</dict>
</plist>
"""

info_plist_path = prefix / "Info.plist"
with open(info_plist_path, "w") as f:
    f.write(info_plist_content)

##############################################

# main.mm
objc_plus_plus_source = """
#import <UIKit/UIKit.h>
#import <zlib.h>
#import <string>
#import <iostream>

@interface AppDelegate : UIResponder <UIApplicationDelegate>
@property (strong, nonatomic) UIWindow *window;
@end

@implementation AppDelegate

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    /*self.window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
    UIViewController *viewController = [[UIViewController alloc] init];
    self.window.rootViewController = viewController;
    
    std::string originalStr = "Hello, Conan and iOS World!";
    uLongf compressedDataSize = compressBound(originalStr.size());
    Bytef *compressedData = (Bytef*)malloc(compressedDataSize);
    
    if (compress(compressedData, &compressedDataSize, (Bytef*)originalStr.data(), originalStr.size()) == Z_OK) {
        std::cout << "Original string: " << originalStr << "\n";
        std::cout << "Compressed size: " << compressedDataSize << "\n";
    } else {
        std::cout << "Compression failed!" << "\n";
    }
    
    free(compressedData);
    
    UILabel *label = [[UILabel alloc] initWithFrame:CGRectMake(20, 100, 300, 40)];
    label.numberOfLines = 0;
    label.text = [NSString stringWithFormat:@"Original: %lu, Compressed: %lu", originalStr.size(), compressedDataSize];
    [viewController.view addSubview:label];
    
    self.window.backgroundColor = [UIColor whiteColor];
    [self.window makeKeyAndVisible];*/
    return YES;
}

@end

int main(int argc, char * argv[]) {
    @autoreleasepool {
        return UIApplicationMain(argc, argv, nil, NSStringFromClass([AppDelegate class]));
    }
}
"""

# Write the source to a file
app_source_file = prefix / "main.mm"
with open(app_source_file, "w") as f:
    f.write(objc_plus_plus_source)

pathtools.ensureDirectoryExists(prefix/".build")
os.chdir(prefix/".build")

the_environ["VERBOSE"] = "1"

print( "############## begin CMakelists.txt ##############")
print(cmake_lists_content)
print( "############## end CMakelists.txt ##############")

print( "############## begin envdump ##############")

for item in the_environ:
  print(f"{item}={the_environ[item]}")

print( "############## end envdump ##############")

command.run([ "cmake", #
              "..", #
              "-GXcode", #
              "-DCMAKE_TOOLCHAIN_FILE=%s"%tc_output, #
              "-DCMAKE_BUILD_TYPE=Debug"], #
              environment=the_environ, #
              do_log=True)

# Build the project with CMake
command.run(["cmake", "--build", ".", "--target", "xros_minimal_app"], environment=the_environ, do_log=True)

# Code sign the app bundle (you may need to adjust this based on your code signing configuration)
#command.run(["codesign", "-s", CERTNAME,  "--deep", app_bundle_dir], do_log=True)


print( "INSTALLING TO SIMULATOR")
XROS_SIM_DEVID = os.environ["XROS_SIM_DEVID"]
command.run(["xcrun","simctl","install",XROS_SIM_DEVID,".build/Debug-xros/xros_minimal_app.app"], 
             working_dir=prefix,
             do_log=True)
command.run(["xcrun","simctl","launch",XROS_SIM_DEVID,".build/Debug-xros/xros_minimal_app.app"], 
             working_dir=prefix,
             do_log=True)

#command.run(["ideviceinstaller","-i",prefix/(BUNDLE_NAME+".app")], do_log=True)



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, tarfile
from obt import dep, host, path, git, cmake, make
from obt.deco import Deco
from obt.wget import wget
from obt.command import Command


deco = Deco()

###############################################################################

class linuxcnc(dep.Provider):

  def __init__(self): ############################################
    super().__init__("linuxcnc")
    self.source_root = path.builds()/"linuxcnc"
    self._archlist = ["x86_64"]

  def provide(self): ##########################################################
    dep.require("boost")
    pfx = path.prefix()
    git.Clone("https://github.com/linuxcnc/linuxcnc",self.source_root,"master")
    os.chdir(self.source_root/"src")
    #os.environ["INSTALL_PREFIX"] = str(path.prefix())
    os.environ["CXXFLAGS"]="-I%s"%(pfx/"include")
    os.environ["LDFLAGS"]="-L%s"%(pfx/"lib")
    os.environ["PKG_CONFIG_PATH"]="%s:/usr/lib/x86_64-linux-gnu/pkgconfig/:/usr/share/pkgconfig"%(pfx/"lib"/"pkgconfig")

    Command(["./autogen.sh"]).exec()

    #Command(["echo","${CXXFLAGS}"]).exec()
    #Command(["echo","${LDFLAGS}"]).exec()
    #Command(["echo","${PKG_CONFIG_PATH}"]).exec()

    Command(["./configure",
             "--with-realtime=uspace",
             "--enable-non-distributable=yes",
             "--with-boost-python=boost_python27-mt",
             "--prefix=%s"%pfx
             ]).exec()

    make.exec("LDFLAGS=${ORK_STAGING_FOLDER}/lib")
    self.manifest.touch()
    self.OK = self.manifest.exists()

    return self.OK



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, tarfile
from obt import dep, host, path

###############################################################################

class shaderc(dep.StdProvider):
  name = "shaderc"
  def __init__(self): ############################################
    super().__init__(shaderc.name)
    self.declareDep("cmake")
    self._builder = self.createBuilder(dep.CMakeBuilder)
    BUILDS = path.builds()
    self.SHADERC_DIR = BUILDS/"shaderc"
    self.SHADERC_SPIRV_TOOLS_DIR = BUILDS/"shaderc"/"external"/"spirv-tools"
    self.SHADERC_SPIRV_HEADERS_DIR = BUILDS/"shaderc"/"external"/"spirv-headers"
    self.SHADERC_GLSLANG_DIR = BUILDS/"shaderc"/"external"/"glslang"
    self._builder.setCmVars({
        "SHADERC_SKIP_TESTS": "ON",
        "SHADERC_SPIRV_TOOLS_DIR": self.SHADERC_SPIRV_TOOLS_DIR,
        "SHADERC_SPIRV_HEADERS_DIR": self.SHADERC_SPIRV_HEADERS_DIR,
        "SHADERC_GLSLANG_DIR": self.SHADERC_GLSLANG_DIR        
    })
  ########################################################################
  def __str__(self): 
    return "zeromq (github-%s)" % VERSION
  ########################################################################
  @property
  def _fetcher(self):
    f = dep.CompositeFetcher("shaderc-composite")
    f1 = dep.GithubFetcher(name="shaderc",
                           repospec="tweakoz/shaderc",
                           revision="v2025.1",
                           recursive=False,
                           dest_dir_override = self.SHADERC_DIR)
    f2 = dep.GithubFetcher(name="shaderc-spirvtools",
                           repospec="tweakoz/SPIRV-Tools",
                           revision="vulkan-sdk-1.4.309.0",
                           recursive=False,
                           dest_dir_override = self.SHADERC_SPIRV_TOOLS_DIR)
    f3 = dep.GithubFetcher(name="shaderc-spirvheaders",
                           repospec="tweakoz/SPIRV-Headers",
                           revision="vulkan-sdk-1.4.309.0",
                           recursive=False,
                           dest_dir_override = self.SHADERC_SPIRV_HEADERS_DIR)
    f4 = dep.GithubFetcher(name="shaderc-glslang",
                           repospec="tweakoz/glslang",
                           revision="vulkan-sdk-1.4.309.0",
                           recursive=False,
                           dest_dir_override = self.SHADERC_GLSLANG_DIR)
    f.addSubFetcher(f1)
    f.addSubFetcher(f2)
    f.addSubFetcher(f3)
    f.addSubFetcher(f4)
    return f 
  ########################################################################
  def linkenv(self):
    LIBS = ["shaderc"]
    return {
        "LIBS": LIBS,
        "LFLAGS": ["-l%s"%item for item in LIBS]
    }
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.includes()/"shaderc"/"shaderc.h").exists()



#!/Users/michael/.venv-apr22/bin/python3

import os

deplist =  ["pkgconfig","wget","curl","libtiff","libpng", "git-lfs"]
deplist += ["portaudio","m4","bison","flex","xz"]
deplist += ["scons","zlib","tbb", "glew","flac","libsndfile"]
deplist += ["mpfr","openssl","graphviz","doxygen","swig","tcl-tk"]
deplist += ["pyqt5","qt5", "gcc@13", "xxhash", "autoconf", "automake" ]

depliststr = " ".join(deplist)
print(depliststr)
os.system("brew install %s" % depliststr)
#os.system("brew remove boost")

os.system("python3 -m pip install --upgrade setuptools")



#!/usr/bin/env python3
###############################################################################
# Orkid Build System
# Copyright 2010-2023, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, sys, argparse, inspect, pathlib, subprocess, multiprocessing, json, importlib, site
import importlib.resources
from functools import lru_cache

Path = pathlib.Path

from obt.deco import Deco 

deco = Deco()

###########################################

def __get_obt_root():
 return importlib.resources.files('obt')

def is_inplace():
  git_folder = __get_obt_root()/".."/".."/".git"
  return os.path.exists(git_folder)

##########################################

@lru_cache(maxsize=None)
def obt_data_base():
  if is_inplace():
    dbase = __get_obt_root()/".."/".."
    return Path(os.path.normpath(dbase))
  else:
    assert("VIRTUAL_ENV" in os.environ)
    p = Path(os.environ["VIRTUAL_ENV"])/"obt"
    assert(p.exists())
    return p
 
##########################################

def __get_modules():
  return obt_data_base()/"modules"

###########################################

def env_is_set(key):
  return str(key) in os.environ and os.environ[str(key)]!=None

def _genpath(inp):
  return pathlib.Path(os.path.realpath(str(inp)))

def _genpaths(inp):
  p = inp.split(":")
  p = [_genpath(x) for x in p]
  # return unique paths, preserving order 
  uset = set()
  rval = []
  for x in p:
    if x not in uset:
      uset.add(x)
      rval.append(x)
  return rval

def _pathlist_to_str(inp):
  as_str = ""
  count = len(inp)
  uset = set()
  for i in range(count):
    item = inp[i]
    item_str = str(item)
    if item_str not in uset:
      if i!=0:
        as_str += ":"
      as_str += item_str
  return as_str

###########################################
# need private copy of InvokingModule fns
###########################################

def _fileOfInvokingModule():
  frame = inspect.stack()[1]
  module = inspect.getmodule(frame[0])
  if module:
    return _genpath(module.__file__)
  else:
      assert(False)

def _directoryOfInvokingModule():
  frame = inspect.stack()[1]
  module = inspect.getmodule(frame[0])
  if module:
    return _genpath(os.path.dirname(module.__file__))
  else:
      assert(False)

###########################################

def _ppath(p):
  return str(p)

def _listToStrList(l):
  return [str(x) for x in l]

def _ppaths(plist):
  rval = ""
  for p in plist:
    rval += "\n\t"+_ppath(p)
  rval += "\n"
  return rval

def findExecutable(exec_name):
  for path_dir in os.environ["PATH"].split(os.pathsep):
    exec_path = os.path.join(path_dir, exec_name)
    if os.path.isfile(exec_path) and os.access(exec_path, os.X_OK):
      return _genpath(exec_path)
  return None

###########################################

def importProject(item):

  project_dirs = []
  
  try_project_manifest = item/"obt.project"/"obt.manifest"

  if try_project_manifest.exists():
    manifest_json = json.load(open(try_project_manifest,"r"))
    project_name = manifest_json["name"]
    #print(manifest_json)
    #config._project_name = manifest_json["name"]
    autoexec = manifest_json["autoexec"]
    autoexec = item/"obt.project"/autoexec
    #print(autoexec)
    assert(autoexec.exists())

    modules = item/"obt.project"/"modules"
    if modules.exists():
      os.environ["OBT_MODULES_PATH"] = os.environ["OBT_MODULES_PATH"] + ":" + str(modules)
    dep = item/"obt.project"/"modules"/"dep"
    if dep.exists():
      os.environ["OBT_DEP_PATH"] = os.environ["OBT_DEP_PATH"] + ":" + str(dep)

    # import autoexec as python module
    spec = importlib.util.spec_from_file_location("autoexec", str(autoexec))
    init_env = importlib.util.module_from_spec(spec) 
    spec.loader.exec_module(init_env)
    print(deco.orange("############################################################################################"))    
    print(deco.orange("Initializing Project: %s"%project_name))
    print(deco.orange("############################################################################################"))    
    init_env.setup()
    print(deco.orange("############################################################################################"))    
    print(deco.orange("Initialized Project: %s"%project_name))
    print(deco.orange("############################################################################################"))    
    if "extend_bashrc" in dir(init_env):
      BASHEXT = init_env.extend_bashrc()
      _config.addBashRcLines(BASHEXT)

###########################################
# Global OBT process execution configuration
#
#  These may be intialized from a couple different paths:
#
#  1. (EXPLICIT) from an OBT script which is run outside of an OBT environment
#       eg obt.env.launch.py, obt.env.create.py
#       in this case the config is initialized from command line arguments
#  2. (IMPLICIT) from OBT scripts running from within an OBT environment
#       eg obt.dep.list.py 
#       in this case
#
#########
#
#  All OBT core path setup should be controlled ONLY by this config
#   and used elsewhere in the OBT codebase. or external OBT based projects
#  Theory of opertation:
#   1. Place all top level configuration into environment
#      variables as early as possible.
#   2. keep state which can be derived soley from env vars as such..
#   3. stack or subspace operations can inherit and modify from upper OBT stack level env vars
#
# ###########################################

###########################################
# The location of this config module is in bin_pub because it needs to be available:
#   1. from outside of an OBT environment 
#   2. AND early in script startup (before sys.paths are setup)
# This also implies this config object cannot import OBT modules itself (self contained)
###########################################

class ObtExecConfig(object):

  #####################################################################

  def __init__(self): # default initialization from env vars

    self._command = None
    self._disable_syspypath = True
    self._git_ssh_command = None
    self._project_dirs = []
    self._bashrc_lines = []
  #####################################################################
  def addBashRcLines(self,lines):
    self._bashrc_lines += lines
  #####################################################################
  @property 
  def num_cores(self):
    return os.environ["OBT_NUM_CORES"]
  #####################################################################
  @property 
  def inplace(self):
    return is_inplace()
  #####################################################################
  @property 
  def quiet(self):
    return env_is_set("OBT_QUIET")
  #####################################################################
  @property 
  def subspace(self):
    if env_is_set("OBT_SUBSPACE"):
      return os.environ["OBT_SUBSPACE"]
    else:
      return None
  #####################################################################
  @property 
  def subspace_dir(self):
    if env_is_set("OBT_SUBSPACE_DIR"):
      return _genpath(os.environ["OBT_SUBSPACE_DIR"])
    else:
      return None
  #####################################################################
  @property 
  def subspace_bin_dir(self):
    return self.subspace_dir/"bin"
  #####################################################################
  @property 
  def subspace_lib_dir(self):
    return self.subspace_dir/"lib"
  #####################################################################
  @property 
  def subspace_build_dir(self):
    return self.subspace_dir/"builds"
  #####################################################################
  @property 
  def subspace_is_conda(self):
    return "conda" in os.environ["OBT_SUBSPACE_DIR"]
  #####################################################################
  @property 
  def text_search_exts(self):
    return os.environ["OBT_SEARCH_EXTLIST"].split(":")
  #####################################################################
  @property 
  def text_search_path(self):
    if env_is_set("OBT_SEARCH_PATH"):
      return _genpaths(os.environ["OBT_SEARCH_PATH"])
    else:
      return []
  #####################################################################
  @property 
  def project_name(self):
    return os.environ["OBT_PROJECT_NAME"]
  #####################################################################
  @property 
  def root_dir(self):
    if env_is_set("OBT_ROOT"):
      return _genpath(os.environ["OBT_ROOT"])
    else:
      return None
  #####################################################################
  @property 
  def scripts_dir(self):
    if env_is_set("OBT_SCRIPTS_DIR"):
      return _genpath(os.environ["OBT_SCRIPTS_DIR"])
    else:
      return None
  #####################################################################
  @property 
  def stage_dir(self):
    if env_is_set("OBT_STAGE"):
      return _genpath(os.environ["OBT_STAGE"])
    else:
      return None
  #####################################################################
  @property 
  def build_dir(self):
    return self.stage_dir/"builds"
  #####################################################################
  @property 
  def original_python_path(self):
    if env_is_set("OBT_ORIGINAL_PYTHONPATH"):
      return _genpaths(os.environ["OBT_ORIGINAL_PYTHONPATH"])
    else:
      return []
  #####################################################################
  @property 
  def python_home(self):
    if env_is_set("OBT_PYTHONHOME"):
      return _genpath(os.environ["OBT_PYTHONHOME"])
    else:
      return None
  #####################################################################
  @property 
  def python_path(self):
    if env_is_set("PYTHONPATH"):
      return _genpaths(os.environ["PYTHONPATH"])
    else:
      return []
  #####################################################################
  @property 
  def original_ld_library_path(self):
    if env_is_set("OBT_ORIGINAL_LD_LIBRARY_PATH"):
      return _genpaths(os.environ["OBT_ORIGINAL_LD_LIBRARY_PATH"])
    else:
      return []
  #####################################################################
  @property 
  def original_pkg_path(self):
    if env_is_set("OBT_ORIGINAL_PKG_CONFIG_PATH"):
      return _genpaths(os.environ["OBT_ORIGINAL_PKG_CONFIG_PATH"])
    else:
      return []
  #####################################################################
  @property 
  def obt_scripts_base():
    if "OBT_SCRIPTS_DIR" in os.environ:
      return Path(os.environ["OBT_SCRIPTS_DIR"])
    else:
      # search sys.path for "obt"
      for item in sys.path:
        obt_subfolder_exists = (_genpath(item)/"obt").exists()
        if obt_subfolder_exists:
          return Path(item)
  #####################################################################
  @property 
  def modules_path(self):
    if env_is_set("OBT_MODULES_PATH"):
      return _genpaths(os.environ["OBT_MODULES_PATH"])
    else:
      return []
  #####################################################################
  @property 
  def dep_path(self):
    if env_is_set("OBT_DEP_PATH"):
      return _genpaths(os.environ["OBT_DEP_PATH"])
    else:
      return []
  #####################################################################
  @property 
  def bin_pub_dir(self):
    if env_is_set("OBT_BIN_PUB_DIR"):
      return _genpath(os.environ["OBT_BIN_PUB_DIR"])
    else:
      return None
  #####################################################################
  @property 
  def bin_priv_dir(self):
    if env_is_set("OBT_BIN_PRIV_DIR"):
      return _genpath(os.environ["OBT_BIN_PRIV_DIR"])
    else:
      return None
  #####################################################################
  @property 
  def python_package_install_dir(self):
    if env_is_set("OBT_PYPKG"):
      return _genpath(os.environ["OBT_PYPKG"])
    else:
      return 0
  #####################################################################
  @property 
  def stack_depth(self):
    if env_is_set("OBT_STACK"):
      return os.environ["OBT_STACK"].count("<")
    else:
      return 0
  #####################################################################

  def log(self,text):
    if self.quiet==False:
      print(text)

  #####################################################################

  def dump_env(self):
    for key in os.environ:
      if "OBT" in key:
        print(key,os.environ[key])

  def dump(self):

    print("##########################################################")
    print( "obtconfig.quiet: %s"%self.quiet )
    print( "obtconfig.inplace: %s"%self.inplace )
    print( "obtconfig.command: %s"%self._command )
    print( "obtconfig.numcores: %s"%self.num_cores )
    print( "obtconfig.stack_depth: %s"%self.stack_depth )
    print("##########################################################")
    print( "obtconfig.project_name: %s"%self.project_name )
    print("##########################################################")
    print( "obtconfig.bin_priv_dir: %s"%_ppath(self.bin_priv_dir ))
    print( "obtconfig.bin_pub_dir: %s"%_ppath(self.bin_pub_dir ))
    print( "obtconfig.scripts_dir: %s"%_ppath(self.scripts_dir ))
    print( "obtconfig.build_dir: %s"%_ppath(self.build_dir ))
    print( "obtconfig.root_dir: %s"%_ppath(self.root_dir ))
    print( "obtconfig.stage_dir: %s"%_ppath(self.stage_dir ))
    print("##########################################################")
    print( "obtconfig.text_search_extensions: %s"%self.text_search_exts )
    print( "obtconfig.text_search_paths: %s"%_ppaths(self.text_search_path ))
    print("##########################################################")
    print( "obtconfig.modules_path: %s"%_ppaths(self.modules_path ))
    print( "obtconfig.dep_path: %s"%_ppaths(self.dep_path ))
    print("##########################################################")
    print( "obtconfig.original_ld_library_path: %s"%_ppaths(self.original_ld_library_path ))
    print( "obtconfig.original_python_paths: %s"%_ppaths(self.original_python_path ))
    print( "obtconfig.current_python_paths: %s"%_ppaths(self.python_path ))
    print( "obtconfig.python_home: %s"%_ppath(self.python_home ))
    print( "obtconfig.python_package_install_dir: %s"%_ppath(self.python_package_install_dir ))
    print("##########################################################")
    print( "obtconfig.subspace: %s"%self.subspace )
    print( "obtconfig.subspace_is_conda: %s"%self.subspace_is_conda )
    print( "obtconfig.subspace_dir: %s"%self.subspace_dir )
    print( "obtconfig.subspace_bin_dir: %s"%self.subspace_bin_dir )
    print( "obtconfig.subspace_lib_dir: %s"%self.subspace_lib_dir )
    print( "obtconfig.subspace_build_dir: %s"%_ppath(self.subspace_build_dir ))
    print("##########################################################")
    print( "obtconfig.VALID: %s"%self.valid )
    print("##########################################################")

  #####################################################################

  @property
  def valid(self):
    valid = (self.bin_priv_dir!=None)
    valid = valid and (self.bin_pub_dir!=None)
    valid = valid and (self.scripts_dir!=None)
    valid = valid and (self.root_dir!=None)
    valid = valid and (self.stage_dir!=None)
    return valid


###############################################################################
#
###############################################################################

global _config
_config = ObtExecConfig() 

def obt_scripts_base():
  if "OBT_SCRIPTS_DIR" in os.environ:
    return Path(os.environ["OBT_SCRIPTS_DIR"])
  else:
    import importlib.util
    obt_spec = importlib.util.find_spec("obt")
    obt_base = Path(obt_spec.submodule_search_locations[0])
    #print(obt_base)
    return obt_base

###########################################
# (EXPLICIT) initialize (bootstrap) from command line arguments
###########################################

def configFromCommandLine(parser_args=None):

  def IS_ARG_SET(arg):
    if (parser_args!=None) and (arg in parser_args) and (parser_args[arg]!=None):
      return True
    else:
      return False

  ########################
  # independent items not attached to directories
  ########################

  if is_inplace():
    os.environ["OBT_INPLACE"] = "1"
    if "OBT_INPLACE" not in os.environ:
      di = _directoryOfInvokingModule()
      obt_folder = di/".."/"scripts/"/"obt"
      if(obt_folder.exists()):
        print("#######################################")
        print("YOU SHOULD USE --inplace if running OBT from a working copy!")
        print("#######################################")
        sys.exit(-1)

  if IS_ARG_SET("quiet") and parser_args["quiet"]:
    os.environ["OBT_QUIET"] = "1"

  if IS_ARG_SET("numcores"):
    numcores = int(parser_args["numcores"])
    os.environ["OBT_NUM_CORES"]=str(numcores)
  else:
    if "OBT_NUM_CORES" not in os.environ:
      numcores = multiprocessing.cpu_count()
      os.environ["OBT_NUM_CORES"]=str(numcores)

  if IS_ARG_SET("prompt"):
    os.environ["OBT_USE_PROMPT_PREFIX"] = parser_args["prompt"]

  os.environ["color_prompt"] = "yes"

  ########################
  # cache exterior env vars
  ########################
  def do_path(a,b):
    if env_is_set(a):
      os.environ[b] = os.environ[a] 
    else:
      os.environ[b] = ""

  ########################
  norm_venvpackages = os.path.normpath(str(site.getsitepackages()[0]))
  norm_pypath = os.path.normpath(str(obt_scripts_base()/".."))
  os.environ["PYTHONPATH"] = norm_venvpackages + ":" + norm_pypath
  os.environ["OBT_VENV_DIR"] = os.environ["VIRTUAL_ENV"]
  os.environ["OBT_VENV_DATA"] = str(obt_data_base())
  ########################

  if not env_is_set("OBT_ORIGINAL_PYTHONPATH"):
    os.environ["OBT_ORIGINAL_PYTHONPATH"] = norm_venvpackages
  
  if not env_is_set("OBT_ROOT"):
    os.environ["OBT_BIN_PUB_DIR"] = str(_directoryOfInvokingModule())
    os.environ["OBT_ROOT"] = str(obt_data_base())

  # find obt.scripts dir using python module search
  if not env_is_set("OBT_SCRIPTS_DIR"):
    os.environ["OBT_SCRIPTS_DIR"] = str(obt_scripts_base())
    os.environ["OBT_BIN_PRIV_DIR"] = str(obt_data_base()/"bin_priv")
   
    #do_path("PYTHONPATH","OBT_ORIGINAL_PYTHONPATH")
    do_path("PATH","OBT_ORIGINAL_PATH")
    do_path("LD_LIBRARY_PATH","OBT_ORIGINAL_LD_LIBRARY_PATH")
    do_path("PS1","OBT_ORIGINAL_PS1")
          
    #################################

    if "PKG_CONFIG" in os.environ:
      os.environ["OBT_ORIGINAL_PKG_CONFIG"] = os.environ["PKG_CONFIG"]
    else :
      orig_pkg_config = findExecutable("pkg-config")
      if orig_pkg_config!=None:
        os.environ["OBT_ORIGINAL_PKG_CONFIG"] = str(orig_pkg_config)
      else:
        print(deco.err("NO PKG-CONFIG FOUND, is your base shell setup correctly ?"))      
        assert(False)

  if("PKG_CONFIG_PATH" not in os.environ):
    orig_pkg_config = findExecutable("pkg-config")
    if orig_pkg_config==None:
      print(deco.err("NO PKG-CONFIG FOUND, is your base shell setup correctly ?"))
      assert(False)
    pkg_config_result = subprocess.run(["pkg-config", "--variable=pc_path", "pkg-config"], capture_output=True, text=True)
    pkg_config_paths = []
    if pkg_config_result.returncode == 0:
      pkg_config_paths = pkg_config_result.stdout.strip().split(':')
      #print(pkg_config_paths)
      os.environ["OBT_ORIGINAL_PKG_CONFIG_PATH"] = ":".join(pkg_config_paths)
      os.environ["PKG_CONFIG_PATH"] = ":".join(pkg_config_paths)
      #print(os.environ)
  else:
    do_path("PKG_CONFIG_PATH","OBT_ORIGINAL_PKG_CONFIG_PATH")

  ########################
  # stage dir
  ########################

  if IS_ARG_SET("stagedir"):
    os.environ["OBT_STAGE"] = os.path.realpath(parser_args["stagedir"])

  assert(env_is_set("OBT_STAGE"))

  ########################
  # project dir(s)
  ########################

  if not env_is_set("OBT_PROJECT_NAME"):
    os.environ["OBT_PROJECT_NAME"] = "OBT"

  if IS_ARG_SET("project"):
    project_dirs = parser_args["project"]
    print(project_dirs)
    _config._project_dirs = _genpaths(":".join(project_dirs))
    os.environ["OBT_PROJECT_DIRS"] = ":".join(project_dirs)

  ########################
  # subspace
  ########################

  if not env_is_set("OBT_SUBSPACE"):
    os.environ["OBT_SUBSPACE"] = "host"
    os.environ["OBT_SUBSPACE_PROMPT"] = "host"
  if IS_ARG_SET("subspace"):
    os.environ["OBT_SUBSPACE"] = parser_args["subspace"]
    os.environ["OBT_SUBSPACE_PROMPT"] = parser_args["subspace"]

  assert(env_is_set("OBT_SUBSPACE"))

  ########################

  if not env_is_set("OBT_MODULES_PATH"):
    os.environ["OBT_MODULES_PATH"] = str(obt_data_base()/"modules")

  ########################

  if not env_is_set("OBT_PYTHONHOME"):
    os.environ["OBT_PYTHONHOME"] = str(_config.stage_dir/"pyvenv")

  ########################

  if IS_ARG_SET("sshkey"):
    GIT_SSH_COMMAND="ssh -i %s" % parser_args["sshkey"]
    _config._git_ssh_command = GIT_SSH_COMMAND

  ########################

  if IS_ARG_SET("command"):
    _config._command = parser_args["command"].split(" ")

  ########################
  # set up sys.path so we can import OBT modules
  ########################

  import obt.env
  import obt.path
  import obt.host
  import obt.dep
  import obt.subspace

  ########################
  print(deco.orange("############################################################################################"))    
  print(deco.orange("Initializing OBT Base"))
  print(deco.orange("############################################################################################"))    
  ########################

  obt.env.append("OBT_DEP_PATH",_config.modules_path[0]/"dep")

  ########################

  if IS_ARG_SET("stack"):
    stack_dir = Path(parser_args["stack"]).resolve()
    obt.env.append("OBT_STACK","<")
    assert(stack_dir.exists())
    assert( stack_dir == _config.stage_dir)

  #####################################################
  # setup os.environfrom config
  #####################################################

  obt.env.prepend("PATH",_config.bin_pub_dir )
  obt.env.prepend("PATH",_config.bin_priv_dir )

  if _config._git_ssh_command!=None:
    obt.env.set("GIT_SSH_COMMAND",_config._git_ssh_command)

  obt.env.set("PYTHONNOUSERSITE","TRUE")

  #####################################################

  if not env_is_set("OBT_SEARCH_EXTLIST"):
    obt.env.set("OBT_SEARCH_EXTLIST",".cpp:.c:.cc:.h:.hpp:.inl:.qml:.m:.mm:.py:.txt:.glfx")
  if not env_is_set("OBT_SEARCH_PATH"):
    os.environ["OBT_SEARCH_PATH"] = ""

  #####################################################
  # items only valid if there is a staging folder
  #####################################################
  
  if _config.valid: # Valid StageDir ?
    obt.env.prepend("PATH",_config.stage_dir/"bin")
    obt.env.prepend("LD_LIBRARY_PATH",_config.stage_dir/"lib")
    obt.env.prepend("LD_LIBRARY_PATH",_config.stage_dir/"lib64")
    obt.env.prepend("PKG_CONFIG",_config.stage_dir/"bin"/"pkg-config")
    obt.env.prepend("PKG_CONFIG_PATH",_config.stage_dir/"lib"/"pkgconfig")
    obt.env.prepend("PKG_CONFIG_PATH",_config.stage_dir/"lib64"/"pkgconfig")
    #obt.env.prepend("PYTHONPATH",_config.stage_dir/"lib"/"python")
    #print(os.environ)
    ########################

    print(deco.orange("############################################################################################"))    
    print(deco.orange("Initialized OBT Base"))
    print(deco.orange("############################################################################################"))    

    for item in _config._project_dirs:
      if item!=_config.root_dir:
        importProject(item)
        #_config.dump_env()

    subspace_dir = _config.stage_dir

    # subspace paths
    if _config.subspace!="host":
      subspace_dir = obt.subspace.descriptor(_config.subspace)._prefix

    def updateSUBS():
      obt.env.set("OBT_SUBSPACE_DIR",str(subspace_dir))
      obt.env.set("OBT_SUBSPACE_LIB_DIR",str(_config.subspace_lib_dir))
      obt.env.set("OBT_SUBSPACE_BIN_DIR",str(_config.subspace_bin_dir))
      obt.env.set("OBT_BUILDS",str(_config.build_dir))
      obt.env.set("OBT_SUBSPACE_BUILD_DIR",str(_config.subspace_build_dir))

    updateSUBS()


  #####################################################
  return _config

###########################################
# (IMPLICIT) default config (from environment)
###########################################

def configFromEnvironment():
  return _config

###########################################
# per dep dynamic env init
###########################################

def initializeDependencyEnvironments(envsetup):

  import obt.host
  import obt.dep
  import obt.sdk
  import obt.subspace

  ####################################
  print(deco.orange("############################################################################################"))    
  print(deco.orange("Initializing Dependencies"))
  print(deco.orange("############################################################################################"))    
  ####################################
  hostinfo = obt.host.description()
  if hasattr(hostinfo,"env_init"):
    hostinfo.env_init()
  ####################################
  sdkitems = obt.sdk.enumerate()
  for sdk_module_key in sdkitems.keys():
    sdk_module_item = sdkitems[sdk_module_key]
    sdk_module = sdk_module_item._module
    sdkinfo = sdk_module.sdkinfo()
    if hasattr(sdkinfo,"env_init"):
      sdkinfo.env_init()
  ####################################
  depitems = obt.dep.DepNode.FindWithMethod("env_init")
  for depitemk in depitems:
    depitem = depitems[depitemk]
    if depitem.supports_host:
      depitem.env_init()
  ####################################
  subspaceitems = obt.subspace.findWithMethod("env_init")
  for subitemk in subspaceitems:
    subitem = subspaceitems[subitemk]
    subitem._module.env_init(envsetup)
  ####################################
  print(deco.orange("############################################################################################"))    
  print(deco.orange("Initialized Dependencies"))
  print(deco.orange("############################################################################################"))    



import sys, os, re, json, argparse, re, shutil
import xml.etree.ElementTree as ET
from xml.dom import minidom

####################################################################

def find_executable(exec_name):
  """Find the executable in PATH or use the provided path."""
  exec_name = os.path.expandvars(exec_name)
  if exec_name.startswith(("./", "../")) or os.path.isabs(exec_name):
    full_path = os.path.abspath(exec_name)
    if os.path.exists(full_path) and os.access(full_path, os.X_OK):
      return full_path
    return None
  for path in os.environ["PATH"].split(os.pathsep):
    full_path = os.path.join(path, exec_name)
    if os.path.exists(full_path) and os.access(full_path, os.X_OK):
      return full_path
  if os.path.exists(exec_name) and os.access(exec_name, os.X_OK):
    return exec_name
  return None

####################################################################

def is_python_script(executable_path):
  """Determine if the given file is a Python script."""
  try:
    with open(executable_path, 'r') as f:
      first_line = f.readline().strip()
      return first_line.startswith("#!") and "python" in first_line
  except Exception as e:
    return False

####################################################################

def orkid_debug_env_vars():
  vars = {
    "ORKID_WORKSPACE_DIR": os.getenv("ORKID_WORKSPACE_DIR"),
    "LD_LIBRARY_PATH": os.getenv("LD_LIBRARY_PATH"),
    "PATH": os.getenv("PATH"),
    "PYTHONPATH": os.getenv("PYTHONPATH"),
    "OBT_STAGE": os.getenv("OBT_STAGE"),
    "ORKID_LEV2_EXAMPLES_DIR": os.getenv("ORKID_LEV2_EXAMPLES_DIR"),
  }
  if "ORKID_GRAPHICS_API" in os.environ:
    vars["ORKID_GRAPHICS_API"] = os.getenv("ORKID_GRAPHICS_API")
  return vars

####################################################################

def xml_prettify(elem):
  """Return a pretty-printed XML string for the Element."""
  rough_string = ET.tostring(elem, 'utf-8')
  reparsed = minidom.parseString(rough_string)
  return reparsed.toprettyxml(indent="  ")

####################################################################

def create_xcode_structure(workspace_path, dap, env_vars, working_dir=None):
  exec_name = dap.executable_name
  bin_path = dap.executable_path
  exec_args = dap.executable_args
  # Remove existing workspace if it exists
  if os.path.exists(workspace_path):
    shutil.rmtree(workspace_path)

  # Create directories
  shared_data_path = os.path.join(workspace_path, "xcshareddata")
  shared_schemes_path = os.path.join(shared_data_path, "xcschemes")

  for directory in [workspace_path, shared_data_path, shared_schemes_path]:
    os.makedirs(directory, exist_ok=True)

  # Create contents.xcworkspacedata
  workspace_content = ET.Element("Workspace", version="1.0")
  workspace_file = os.path.join(workspace_path, 'contents.xcworkspacedata')
  with open(workspace_file, 'w') as f:
    f.write(xml_prettify(workspace_content))

  # Generate .xcscheme file
  scheme_name = os.path.basename(bin_path)
  scheme_file = os.path.join(shared_schemes_path, f'{scheme_name}.xcscheme')
  scheme_content = ET.Element("Scheme", LastUpgradeVersion="1500", version="1.7")

  launch_action = ET.SubElement(scheme_content, "LaunchAction",
                                buildConfiguration="Debug",
                                selectedDebuggerIdentifier="Xcode.DebuggerFoundation.Debugger.LLDB",
                                selectedLauncherIdentifier="Xcode.DebuggerFoundation.Launcher.LLDB",
                                launchStyle="0", 
                                useCustomWorkingDirectory="YES" if (working_dir!=None) else "NO",
                                ignoresPersistentStateOnLaunch="NO",
                                debugDocumentVersioning="YES", debugServiceExtension="internal",
                                allowLocationSimulation="YES")

  if working_dir!=None:
    launch_action.set('customWorkingDirectory', working_dir)

  path_runnable = ET.SubElement(launch_action, "PathRunnable", runnableDebuggingMode="0", FilePath=bin_path)

  env_vars_elem = ET.SubElement(launch_action, "EnvironmentVariables")
  for key, value in env_vars.items():
    if value:
      ET.SubElement(env_vars_elem, "EnvironmentVariable", key=key, value=value, isEnabled="YES")

  # Integrate exec_args into the scheme
  ET.SubElement(launch_action, "CommandLineArguments").extend(
    [ET.Element("CommandLineArgument", argument=arg, isEnabled="YES") for arg in exec_args]
  )

  with open(scheme_file, 'w') as f:
    f.write(xml_prettify(scheme_content))

  os.system(f"open {workspace_path}")
  
  
class DebugArgParser:
  #########################################
  def __init__(self):
    self.executable_name = sys.argv[1]
    if len(sys.argv) > 2:
      self.executable_args = sys.argv[2:]
    else:
      self.executable_args = []
    self._compute()
    #exe_path, exe_args, exe_name = get_exec_and_args(args)
  #########################################
  def _compute(self):
    self.executable_path = find_executable(self.executable_name)

    if not self.executable_path:
      print(f"Executable '{self.executable_name}' not found in $PATH.")
      exit(1)

    if is_python_script(self.executable_path):
      python_path = find_executable("python3")
      if not python_path:
        print("python3 not found.")
        exit(1)
      self.executable_args.insert(0, self.executable_path)
      self.executable_path = os.path.realpath(python_path)

    self.exec_name = re.sub(r"[^a-zA-Z0-9]", "_", self.executable_name)
    #return executable_path, executable_args, exec_name

  #########################################

#env_vars = dh.orkid_debug_env_vars()



#!/usr/bin/env python3 

import argparse, importlib, pkgutil, sys, pathlib, ast
from obt.deco import Deco 

deco = Deco() 

parser = argparse.ArgumentParser(description='Enumerate the contents of a namespace package')
parser.add_argument('ns_name', type=str, help='The name of the namespace package to enumerate')
args = parser.parse_args()

###############################################################################

module_colors = [ # blue
  (64,64,128),
  (192,192,224),
  (224,224,255)
]
package_colors = [ # green
  (64,128,64),
  (192,224,192),
  (224,255,224)
]
method_colors = [ # magenta
  (128,64,128),
  (224,192,224),
  (255,224,255)
]
function_colors = [ # yellow-green
  (128,128,64),
  (224,224,192),
  (255,255,224)
]
class_colors = [ # cyan
  (64,128,128),
  (192,224,224),
  (224,255,255)
]

###############################################################################

def iterate_module_contents(item_path : pathlib.Path, level: int):
  indent = '  ' * level
  short_name = item_path.parts[-1]
  mod_color = module_colors[level]
  colored_mod = deco.rgbstr(*mod_color, short_name)
  colored_state = deco.yellow('module')
  print(f'{indent}{colored_mod} : {colored_state}')
  # enumerate the contents of the module (without importing it)
  # this is done by reading the file and looking for methods
  # we can use the ast module to parse the file
  # and extract the methods
  # for now, we just print the methods and classes
  #print(item_path)
  dot_py = item_path.with_suffix('.py')
  visited_functions = set()
  if dot_py.exists() and dot_py.is_file():
    the_ast = ast.parse(open(dot_py).read())
    # first walk classes, and their methods (recusively)
    for node in ast.walk(the_ast):
      is_class = isinstance(node, ast.ClassDef)
      if is_class:
        class_name = node.name
        class_color = class_colors[level]
        colored_class = deco.rgbstr(*class_color, class_name)
        colored_state = deco.yellow('class')
        print(f'{indent}  {colored_class} : {colored_state}')
        for node in ast.walk(node):
          is_function = isinstance(node, ast.FunctionDef)
          if is_function:
            visited_functions.add(node)
            method_name = node.name
            method_color = method_colors[level]
            colored_method = deco.rgbstr(*method_color, method_name)
            colored_state = deco.yellow('cfn')
            print(f'{indent}    {colored_method} : {colored_state}')
    # now walk the global methods
    for node in ast.walk(the_ast):
      is_function = isinstance(node, ast.FunctionDef)
      if is_function:
        if node not in visited_functions:
          fn_name = node.name
          fn_color = function_colors[level]
          colored_method = deco.rgbstr(*fn_color, fn_name)
          colored_state = deco.yellow('gfn')
          print(f'{indent}  {colored_method} : {colored_state}')

         

###############################################################################

def iterate_package_contents(item_path : pathlib.Path, level: int):
  indent = '  ' * level
  short_name = item_path.parts[-1]
  pkgcolor = package_colors[level]
  colored_pkg = deco.rgbstr(*pkgcolor,short_name)
  colored_state = deco.yellow('package')
  print(f'{indent}{colored_pkg} : {colored_state}')
  # recursively enumerate the contents of the package
  for importer, modname, ispkg in pkgutil.iter_modules([str(item_path)]):
    if ispkg:
      iterate_package_contents(item_path / modname, level + 1)
    else:
      iterate_module_contents(item_path / modname, level + 1)

###############################################################################

def directory(ns_name : str):
  # enumerate all namespace packages in PYTHON_PATH with the given name "ns_name"
  # find all packages named "ns_name" in the PYTHON_PATH
  
  module_records_by_path = []
  
  # search for all namespace packages in PYTHON_PATH with the given name "ns_name"
  
  paths = sys.path
  for path in paths:
    as_p = pathlib.Path(path)
    ns_test = as_p / ns_name
    # check if the path has scripts as parent (via last part of path)
    if(len(as_p.parts)) > 0:
      has_scripts_as_parent = as_p.parts[-1] == 'scripts'
      if ns_test.exists() and has_scripts_as_parent:
        # check for namespace packages
        init_scr = ns_test / '__init__.py'
        if not init_scr.exists():
          # this is a namespace package
          print(f'Namespace package found: {ns_test}')
          module_records_by_path.append(ns_test)

  # now we have all namespace packages in PYTHON_PATH with the given name "ns_name"
  # we can enumerate the contents of each package (without importing it)
  # recursively enumerate the contents of each package
  # if a module has methods, enumerate them too
  for ns_path in module_records_by_path:
    colored_path = deco.path(ns_path)
    print(deco.val(f'Enumerating contents of namespace package: {colored_path}'))
    for importer, modname, ispkg in pkgutil.iter_modules([str(ns_path)]):
      if ispkg:
        iterate_package_contents(ns_path / modname, 0)
      else:
        iterate_module_contents(ns_path / modname, 0)


directory(args.ns_name)



#!/usr/bin/env python3
import jinja2, git, datetime, os, subprocess, argparse, shlex, logging, sys, threading, time
from http import server
from yarl import URL
from pathlib import Path
import uuid, urllib

SCRIPTDIR=Path(os.path.dirname(os.path.realpath(__file__)))

################################################################################

def HandlerClass(OUTPUTDIR,SITELOCK):
  class handlerclass(server.SimpleHTTPRequestHandler):
      ###############################################################
      server_version = "yo"
      sys_version = "what up"
      _OUTPUTDIR = OUTPUTDIR
      _SITELOCK = SITELOCK
      ###############################################################
      def __init__(self,req,client_addr,server):
       self.outputdir = handlerclass._OUTPUTDIR
       self.sitelock = handlerclass._SITELOCK
       self.errtext = "YO".encode("UTF-8")
       super(handlerclass, self).__init__(req,client_addr,server)
      ###############################################################
      def send_my_headers(self):
        self.send_header("Cache-Control", "no-cache")
        self.send_header("Pragma", "no-cache")
        self.send_header("Expires", "0")
        self.send_header("ETag", '"'+str(uuid.uuid4())+'"')
      ###############################################################
      def end_headers(self):
        self.send_my_headers()
        server.SimpleHTTPRequestHandler.end_headers(self)
      ###############################################################
      def do_ERROR(self):
        self.send_response(200)
        self.send_header("Content-type", "text/html")
        self.send_header("Content-length", len(self.errtext))
        self.end_headers()
        self.wfile.write(self.errtext)
      ###############################################################
      def translate_path(self, path):
        path = path.strip("/")
        print("inppath<%s>"%(path))
        if path == "":
          path = "index.html"
        elif False==os.path.isfile(self.outputdir/path):
          path = "ERROR"
        qualifiedurl = self.outputdir/path
        print("path<%s> qual<%s>"%(path,qualifiedurl))
        return str(qualifiedurl)
      ###############################################################
      def do_GET(self):
        rval = None
        self.sitelock.acquire(blocking=True)
        referer = self.headers.get('Referer')
        print("Referer<%s> path<%s>"%(referer,self.path))
        self.path = self.path.strip("/")
        if self.path == "":
          self.path = "index.html"
        if os.path.isfile(self.outputdir/self.path):
          rval = server.SimpleHTTPRequestHandler.do_GET(self)
        else:
          rval = None
          self.do_ERROR()
        self.sitelock.release()
        return rval
      ###############################################################
  return handlerclass



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from obt import dep, log, path, template
###############################################################################
class usd(dep.StdProvider):
  name = "usd"
  def __init__(self):
    super().__init__(usd.name)
    python = self.declareDep("python")
    self.declareDep("boost")
    self.declareDep("llvm")
    self.declareDep("opensubdiv")
    self.declareDep("oiio")
    self.declareDep("osl")
    self.declareDep("ptex")
    self.declareDep("pyside2")
    self.declareDep("pyopengl")
    self._builder = self.createBuilder(dep.CMakeBuilder)
    self._builder._cmakeenv = {
      "CMAKE_CXX_STANDARD": "17",
      "PXR_ENABLE_PYTHON_SUPPORT": "ON",
      "PXR_USE_PYTHON_3": "ON",
      "PXR_ENABLE_GL_SUPPORT": "ON",
      #"PXR_ENABLE_VULKAN_SUPPORT": "ON",
      "PXR_ENABLE_OSL_SUPPORT": "ON",
      "BOOST_ROOT": path.stage(),
      "Boost_NO_SYSTEM_PATHS": "ON",
      "CMAKE_LIBRARY_PATH": python.library_dir,
      "CMAKE_SHARED_LINKER_FLAGS": "-L"+str(python.library_dir),
      "CMAKE_MODULE_LINKER_FLAGS": "-L"+str(python.library_dir),
      "CMAKE_EXE_LINKER_FLAGS": "-L"+str(python.library_dir),
      "PYTHON_INCLUDE_DIR": python.include_dir,
      "PYTHON_LIBRARY": python.library_dir/python.library_file,
      "PYTHON_LIBRARY_DEBUG": python.library_dir/python.library_file,
      #"Boost_DEBUG":"ON"
    }

  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=usd.name,
                             repospec="PixarAnimationStudios/USD",
                             revision="release",
                             recursive=True)

  ########################################################################

  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"libusd_arch.so").exists()
###############################################################################


